classdef Interact_exported < matlab.apps.AppBase

    % Properties that correspond to app components
    properties (Access = public)
        UIFigure                     matlab.ui.Figure
        GridLayout                   matlab.ui.container.GridLayout
        LeftPanel                    matlab.ui.container.Panel
        GoZeroButton                 matlab.ui.control.Button
        RunningLampLabel             matlab.ui.control.Label
        RunningLamp                  matlab.ui.control.Lamp
        StepSizeSliderLabel          matlab.ui.control.Label
        StepSizeSlider               matlab.ui.control.Slider
        GoButton                     matlab.ui.control.Button
        StopButton                   matlab.ui.control.Button
        XButton                      matlab.ui.control.Button
        XButton_2                    matlab.ui.control.Button
        YButton                      matlab.ui.control.Button
        YButton_2                    matlab.ui.control.Button
        ZButton                      matlab.ui.control.Button
        ZButton_2                    matlab.ui.control.Button
        ConnectButton                matlab.ui.control.StateButton
        GoHomeButton                 matlab.ui.control.Button
        Dim3LengthZEditField_2Label  matlab.ui.control.Label
        Dim3LengthZEditField         matlab.ui.control.NumericEditField
        Dim2MinorRYEditField_2Label  matlab.ui.control.Label
        Dim2MinorRYEditField         matlab.ui.control.NumericEditField
        Dim1MajorRXEditField_2Label  matlab.ui.control.Label
        Dim1MajorRXEditField         matlab.ui.control.NumericEditField
        GeometryButtonGroup          matlab.ui.container.ButtonGroup
        TubeButton                   matlab.ui.control.RadioButton
        BoxButton                    matlab.ui.control.RadioButton
        XEditFieldLabel              matlab.ui.control.Label
        XEditField                   matlab.ui.control.NumericEditField
        YEditFieldLabel              matlab.ui.control.Label
        YEditField                   matlab.ui.control.NumericEditField
        ZEditFieldLabel              matlab.ui.control.Label
        ZEditField                   matlab.ui.control.NumericEditField
        JumpToGlobalLabel            matlab.ui.control.Label
        SetHomeButton                matlab.ui.control.Button
        SetZeroButton                matlab.ui.control.Button
        JumpButton                   matlab.ui.control.Button
        RightPanel                   matlab.ui.container.Panel
        UIAxes_1                     matlab.ui.control.UIAxes
        UIAxes_2                     matlab.ui.control.UIAxes
    end

    % Properties that correspond to apps with auto-reflow
    properties (Access = private)
        onePanelWidth = 576;
    end

    
    properties (Access = private)
        Machine; % Description
        Home = [0,0,0]; % should be top front left corner
        Location = [0,0,0];
        StepSize = 0.5; % one stepper motor step size in mm (change later)
        StepRoughness = 1; % number of steps taken in each manual moving action
        StopFlag = 0;
        Zero = [0,0,0];
        WaitPeriod = 0.75;
        MarkerSize = 6;
    end
    

    % Callbacks that handle component events
    methods (Access = private)

        % Button pushed function: StopButton
        function StopPushed(app, event)
            app.StopFlag = 1;
            app.RunningLamp.Color = [0.8,0.8,0.8];
        end

        % Button pushed function: GoButton
        function GoPushed(app, event)
            app.RunningLamp.Color = [0,1,0];
            [X,Y,Z,Bx,By,Bz] = deal([]);
            if(app.TubeButton.Value)
                MajR = app.Dim1MajorRXEditField.Value;
                MinR = app.Dim2MinorRYEditField.Value;
                L = app.Dim3LengthZEditField.Value;
                
                for z = 0:app.StepSize*app.StepRoughness:L
                    for r = MajR:-app.StepSize*app.StepRoughness:MinR
                        for theta = 0:app.StepSize*app.StepRoughness/r:2*pi
                            X = [X r*cos(theta)];
                            Y = [Y r*sin(theta)];
                            Z = [Z z];
                        end
                    end
                end   
            else
                W = app.Dim1MajorRXEditField.Value;
                D = app.Dim2MinorRYEditField.Value;
                H = app.Dim3LengthZEditField.Value; 
                
                for z = 0:app.StepSize*app.StepRoughness:H
                    for w = W:-2*app.StepSize*app.StepRoughness:0
                        X = [X w/2.*ones(1,length(-w/2:app.StepSize*app.StepRoughness:w/2)),...
                            w/2:-app.StepSize*app.StepRoughness:-w/2,...
                            -w/2.*ones(1,length(w/2:-app.StepSize*app.StepRoughness:-w/2)),...
                            -w/2:app.StepSize*app.StepRoughness:w/2];
                        Y = [Y -w/2:app.StepSize*app.StepRoughness:w/2,...
                            w/2.*ones(1,length(w/2:-app.StepSize*app.StepRoughness:-w/2)),...
                            w/2:-app.StepSize*app.StepRoughness:-w/2,...
                            -w/2.*ones(1,length(-w/2:app.StepSize*app.StepRoughness:w/2))];
                        Z = [Z z.*ones(1,length(Y)-length(Z))];
                    end
                end
            end
            
            % Go to first point
            numXSteps = round((X(1)-app.Location(1)+app.Zero(1))/app.StepSize);
            numYSteps = round((Y(1)-app.Location(2)+app.Zero(2))/app.StepSize); 
            numZSteps = round((Z(1)-app.Location(3)+app.Zero(3))/app.StepSize);
            writeline(app.Machine,"X"+num2str(numXSteps)+"Y"+num2str(numYSteps)+"Z0");
            readline(app.Machine);
            writeline(app.Machine,"X0Y0"+"Z"+num2str(numZSteps));
            readline(app.Machine);
            app.Location = app.Location+app.StepSize.*[numXSteps,numYSteps,numZSteps];
            disp(app.Location);
            pause(app.WaitPeriod);
            
            for i = 1:length(X)
                if(app.StopFlag == 0)
                    numXSteps = round((X(i)-app.Location(1)+app.Zero(1))/app.StepSize);
                    numYSteps = round((Y(i)-app.Location(2)+app.Zero(2))/app.StepSize); 
                    numZSteps = round((Z(i)-app.Location(3)+app.Zero(3))/app.StepSize);
                    writeline(app.Machine,"X"+num2str(numXSteps)+"Y"+num2str(numYSteps)+"Z"+num2str(numZSteps));
                    data = char(readline(app.Machine));
                    RD = str2num(data(1:end-1));
                    Bx = [Bx -RD(2)];
                    By = [By RD(3)];
                    Bz = [Bz RD(1)];
                    app.Location = app.Location+app.StepSize.*[numXSteps,numYSteps,numZSteps];
                    disp(app.Location);
                    pause(app.WaitPeriod);
                else
                    app.StopFlag = 0;
                    break;
                end
            end
            
            % Plot
            ploti = min([length(Bx),length(By),length(Bz)]);
            quiver3(app.UIAxes_2,X(1:ploti),Y(1:ploti),Z(1:ploti),Bx(1:ploti),By(1:ploti),Bz(1:ploti));
            app.RunningLamp.Color = [0.8,0.8,0.8];
        end

        % Button pushed function: GoHomeButton
        function GoHomePushed(app, event)
            xval = app.Home(1);
            yval = app.Home(2);
            zval = app.Home(3);
            numXSteps = round((xval-app.Location(1))/app.StepSize);
            numYSteps = round((yval-app.Location(2))/app.StepSize); 
            numZSteps = round((zval-app.Location(3))/app.StepSize); 
            writeline(app.Machine,"X0Y0"+"Z"+num2str(numZSteps));
            readline(app.Machine);
            writeline(app.Machine,"X"+num2str(numXSteps)+"Y"+num2str(numYSteps)+"Z0");
            readline(app.Machine);
            app.Location = app.Location+app.StepSize.*[numXSteps,numYSteps,numZSteps];
            disp(app.Location);
        end

        % Button pushed function: GoZeroButton
        function GoZeroPushed(app, event)
            xval = app.Zero(1);
            yval = app.Zero(2);
            zval = app.Zero(3);
            numXSteps = round((xval-app.Location(1))/app.StepSize);
            numYSteps = round((yval-app.Location(2))/app.StepSize); 
            numZSteps = round((zval-app.Location(3))/app.StepSize);
            writeline(app.Machine,"X"+num2str(numXSteps)+"Y"+num2str(numYSteps)+"Z0");
            readline(app.Machine);
            writeline(app.Machine,"X0Y0"+"Z"+num2str(numZSteps));
            readline(app.Machine);
            app.Location = app.Location+app.StepSize.*[numXSteps,numYSteps,numZSteps];
            disp(app.Location);
        end

        % Value changed function: StepSizeSlider
        function SliderChanged(app, event)
            value = app.StepSizeSlider.Value;
            StepOptions = 1:80;
            [~,i] = min(abs((StepOptions./max(StepOptions)-value)));
            app.StepRoughness = StepOptions(i);
            
            if(app.TubeButton.Value&&(app.StepRoughness>3))
                MajR = app.Dim1MajorRXEditField.Value;
                MinR = app.Dim2MinorRYEditField.Value;
                L = app.Dim3LengthZEditField.Value;
                
                [X,Y,Z] = deal([]);
                
                for z = 0:app.StepSize*app.StepRoughness:L
                    for r = MajR:-app.StepSize*app.StepRoughness:MinR
                        for theta = 0:app.StepSize*app.StepRoughness/r:2*pi
                            X = [X r*cos(theta)];
                            Y = [Y r*sin(theta)];
                            Z = [Z z];
                        end
                    end
                end
                scatter3(app.UIAxes_1,X,Y,Z,app.MarkerSize,'filled');
            elseif(app.BoxButton.Value&&(app.StepRoughness>3))
                W = app.Dim1MajorRXEditField.Value;
                D = app.Dim2MinorRYEditField.Value;
                H = app.Dim3LengthZEditField.Value; 
                
                [X,Y,Z] = deal([]);
                
                for z = 0:app.StepSize*app.StepRoughness:H
                    for w = W:-2*app.StepSize*app.StepRoughness:0
                        X = [X w/2.*ones(1,length(-w/2:app.StepSize*app.StepRoughness:w/2)),...
                            w/2:-app.StepSize*app.StepRoughness:-w/2,...
                            -w/2.*ones(1,length(w/2:-app.StepSize*app.StepRoughness:-w/2)),...
                            -w/2:app.StepSize*app.StepRoughness:w/2];
                        Y = [Y -w/2:app.StepSize*app.StepRoughness:w/2,...
                            w/2.*ones(1,length(w/2:-app.StepSize*app.StepRoughness:-w/2)),...
                            w/2:-app.StepSize*app.StepRoughness:-w/2,...
                            -w/2.*ones(1,length(-w/2:app.StepSize*app.StepRoughness:w/2))];
                        Z = [Z z.*ones(1,length(Y)-length(Z))];
                    end
                end
                scatter3(app.UIAxes_1,X,Y,Z,app.MarkerSize,'filled');
            end
        end
        
        % Button pushed function: XButton
        function PXPushed(app, event)
            writeline(app.Machine,"X"+num2str(app.StepRoughness)+"Y0Z0");
            readline(app.Machine);
            app.Location = app.Location+app.StepSize.*[app.StepRoughness,0,0];
            disp(app.Location);
        end

        % Button pushed function: XButton_2
        function NXPushed(app, event)
            writeline(app.Machine,"X"+num2str(-1*app.StepRoughness)+"Y0Z0");
            readline(app.Machine);
            app.Location = app.Location+app.StepSize.*[-app.StepRoughness,0,0];
            disp(app.Location);
        end
        
        % Button pushed function: YButton
        function PYPushed(app, event)
            writeline(app.Machine,"X0Y"+num2str(app.StepRoughness)+"Z0");
            readline(app.Machine);
            app.Location = app.Location+app.StepSize.*[0,app.StepRoughness,0];
            disp(app.Location);
        end
        
        % Button pushed function: YButton_2
        function NYPushed(app, event)
            writeline(app.Machine,"X0Y"+num2str(-1*app.StepRoughness)+"Z0");
            readline(app.Machine);
            app.Location = app.Location+app.StepSize.*[0,-app.StepRoughness,0];
            disp(app.Location);
        end

        % Button pushed function: ZButton
        function PZPushed(app, event)
            writeline(app.Machine,"X0Y0Z"+num2str(app.StepRoughness));
            readline(app.Machine);
            app.Location = app.Location+app.StepSize.*[0,0,app.StepRoughness];
            disp(app.Location);
        end

        % Button pushed function: ZButton_2
        function NZPushed(app, event)
            writeline(app.Machine,"X0Y0Z"+num2str(-1*app.StepRoughness));
            readline(app.Machine);
            app.Location = app.Location+app.StepSize.*[0,0,-app.StepRoughness];   
            disp(app.Location);
        end

        % Selection changed function: GeometryButtonGroup
        function GeometryChanged(app, event)
            if(app.TubeButton.Value)
%                 MajR = app.Dim1MajorRXEditField.Value;
%                 MinR = app.Dim2MinorRYEditField.Value;
%                 L = app.Dim3LengthZEditField.Value;
%                 
%                 [X,Y,Z] = cylinder(app.UIAxes_1,MajR,200);
%                 surf(app.UIAxes_1,X,Y,L.*Z,'EdgeColor','none','FaceAlpha',0.55,'FaceColor',[0,1,0]);
%                 hold(app.UIAxes_1,'on');
%                 plot3(app.UIAxes_1,X,Y,L.*ones(1,length(X)),'k');
%                 plot3(app.UIAxes_1,X,Y,zeros(1,length(X)),'k');
%                 [X,Y,Z] = cylinder(app.UIAxes_1,MinR,200);
%                 surf(app.UIAxes_1,X,Y,L.*Z,'EdgeColor','none','FaceAlpha',0.7,'FaceColor',[0,.92,0]);
%                 plot3(app.UIAxes_1,X,Y,L.*ones(1,length(X)),'k');
%                 plot3(app.UIAxes_1,X,Y,zeros(1,length(X)),'k');
%                 hold(app.UIAxes_1,'off');

                MajR = app.Dim1MajorRXEditField.Value;
                MinR = app.Dim2MinorRYEditField.Value;
                L = app.Dim3LengthZEditField.Value;
                
                [X,Y,Z] = deal([]);
                
                for z = 0:app.StepSize*app.StepRoughness:L
                    for r = MajR:-app.StepSize*app.StepRoughness:MinR
                        for theta = 0:app.StepSize*app.StepRoughness/r:2*pi
                            X = [X r*cos(theta)];
                            Y = [Y r*sin(theta)];
                            Z = [Z z];
                        end
                    end
                end
                scatter3(app.UIAxes_1,X,Y,Z,app.MarkerSize,'filled');
            else
%                 W = app.Dim1MajorRXEditField.Value;
%                 D = app.Dim2MinorRYEditField.Value;
%                 H = app.Dim3LengthZEditField.Value;
%                 
%                 a = -pi:pi/2:pi; % Define Corners
%                 ph = pi/4; % Define Angular Orientation (‘Phase’)
%                 x = W.*[cos(a+ph); cos(a+ph)]/cos(ph);
%                 y = D.*[sin(a+ph); sin(a+ph)]/sin(ph);
%                 z = (H/2).*[-ones(size(a)); ones(size(a))]+H/2;
% 
%                 surf(app.UIAxes_1,x,y,z,'FaceColor','g','FaceAlpha',0.7); % Plot Cube
%                 hold(app.UIAxes_1,'on');
%                 patch(app.UIAxes_1,x',y',z','g','FaceAlpha',0.7); % Make Cube Appear Solid
%                 hold(app.UIAxes_1,'off');
                W = app.Dim1MajorRXEditField.Value;
                D = app.Dim2MinorRYEditField.Value;
                H = app.Dim3LengthZEditField.Value; 
                
                [X,Y,Z] = deal([]);
                
                for z = 0:app.StepSize*app.StepRoughness:H
                    for w = W:-2*app.StepSize*app.StepRoughness:0
                        X = [X w/2.*ones(1,length(-w/2:app.StepSize*app.StepRoughness:w/2)),...
                            w/2:-app.StepSize*app.StepRoughness:-w/2,...
                            -w/2.*ones(1,length(w/2:-app.StepSize*app.StepRoughness:-w/2)),...
                            -w/2:app.StepSize*app.StepRoughness:w/2];
                        Y = [Y -w/2:app.StepSize*app.StepRoughness:w/2,...
                            w/2.*ones(1,length(w/2:-app.StepSize*app.StepRoughness:-w/2)),...
                            w/2:-app.StepSize*app.StepRoughness:-w/2,...
                            -w/2.*ones(1,length(-w/2:app.StepSize*app.StepRoughness:w/2))];
                        Z = [Z z.*ones(1,length(Y)-length(Z))];
                    end
                end
                scatter3(app.UIAxes_1,X,Y,Z,app.MarkerSize,'filled');
            end
        end

        % Value changed function: Dim1MajorRXEditField
        function Dim1Changed(app, event)
            if(app.TubeButton.Value)
%                 MajR = app.Dim1MajorRXEditField.Value;
%                 MinR = app.Dim2MinorRYEditField.Value;
%                 L = app.Dim3LengthZEditField.Value;
%                 
%                 [X,Y,Z] = cylinder(app.UIAxes_1,MajR,200);
%                 surf(app.UIAxes_1,X,Y,L.*Z,'EdgeColor','none','FaceAlpha',0.55,'FaceColor',[0,1,0]);
%                 hold(app.UIAxes_1,'on');
%                 plot3(app.UIAxes_1,X,Y,L.*ones(1,length(X)),'k');
%                 plot3(app.UIAxes_1,X,Y,zeros(1,length(X)),'k');
%                 [X,Y,Z] = cylinder(app.UIAxes_1,MinR,200);
%                 surf(app.UIAxes_1,X,Y,L.*Z,'EdgeColor','none','FaceAlpha',0.7,'FaceColor',[0,.92,0]);
%                 plot3(app.UIAxes_1,X,Y,L.*ones(1,length(X)),'k');
%                 plot3(app.UIAxes_1,X,Y,zeros(1,length(X)),'k');
%                 hold(app.UIAxes_1,'off');

                MajR = app.Dim1MajorRXEditField.Value;
                MinR = app.Dim2MinorRYEditField.Value;
                L = app.Dim3LengthZEditField.Value;
                
                [X,Y,Z] = deal([]);
                
                for z = 0:app.StepSize*app.StepRoughness:L
                    for r = MajR:-app.StepSize*app.StepRoughness:MinR
                        for theta = 0:app.StepSize*app.StepRoughness/r:2*pi
                            X = [X r*cos(theta)];
                            Y = [Y r*sin(theta)];
                            Z = [Z z];
                        end
                    end
                end
                scatter3(app.UIAxes_1,X,Y,Z,app.MarkerSize,'filled');
            else
%                 W = app.Dim1MajorRXEditField.Value;
%                 D = app.Dim2MinorRYEditField.Value;
%                 H = app.Dim3LengthZEditField.Value;
%                 
%                 a = -pi:pi/2:pi; % Define Corners
%                 ph = pi/4; % Define Angular Orientation (‘Phase’)
%                 x = W.*[cos(a+ph); cos(a+ph)]/cos(ph);
%                 y = D.*[sin(a+ph); sin(a+ph)]/sin(ph);
%                 z = (H/2).*[-ones(size(a)); ones(size(a))]+H/2;
% 
%                 surf(app.UIAxes_1,x,y,z,'FaceColor','g','FaceAlpha',0.7); % Plot Cube
%                 hold(app.UIAxes_1,'on');
%                 patch(app.UIAxes_1,x',y',z','g','FaceAlpha',0.7); % Make Cube Appear Solid
%                 hold(app.UIAxes_1,'off');
                W = app.Dim1MajorRXEditField.Value;
                D = app.Dim2MinorRYEditField.Value;
                H = app.Dim3LengthZEditField.Value; 
                
                [X,Y,Z] = deal([]);
                
                for z = 0:app.StepSize*app.StepRoughness:H
                    for w = W:-2*app.StepSize*app.StepRoughness:0
                        X = [X w/2.*ones(1,length(-w/2:app.StepSize*app.StepRoughness:w/2)),...
                            w/2:-app.StepSize*app.StepRoughness:-w/2,...
                            -w/2.*ones(1,length(w/2:-app.StepSize*app.StepRoughness:-w/2)),...
                            -w/2:app.StepSize*app.StepRoughness:w/2];
                        Y = [Y -w/2:app.StepSize*app.StepRoughness:w/2,...
                            w/2.*ones(1,length(w/2:-app.StepSize*app.StepRoughness:-w/2)),...
                            w/2:-app.StepSize*app.StepRoughness:-w/2,...
                            -w/2.*ones(1,length(-w/2:app.StepSize*app.StepRoughness:w/2))];
                        Z = [Z z.*ones(1,length(Y)-length(Z))];
                    end
                end
                scatter3(app.UIAxes_1,X,Y,Z,app.MarkerSize,'filled');
            end
        end

        % Value changed function: Dim2MinorRYEditField
        function Dim2Changed(app, event)
            if(app.TubeButton.Value)
%                 MajR = app.Dim1MajorRXEditField.Value;
%                 MinR = app.Dim2MinorRYEditField.Value;
%                 L = app.Dim3LengthZEditField.Value;
%                 
%                 [X,Y,Z] = cylinder(app.UIAxes_1,MajR,200);
%                 surf(app.UIAxes_1,X,Y,L.*Z,'EdgeColor','none','FaceAlpha',0.55,'FaceColor',[0,1,0]);
%                 hold(app.UIAxes_1,'on');
%                 plot3(app.UIAxes_1,X,Y,L.*ones(1,length(X)),'k');
%                 plot3(app.UIAxes_1,X,Y,zeros(1,length(X)),'k');
%                 [X,Y,Z] = cylinder(app.UIAxes_1,MinR,200);
%                 surf(app.UIAxes_1,X,Y,L.*Z,'EdgeColor','none','FaceAlpha',0.7,'FaceColor',[0,.92,0]);
%                 plot3(app.UIAxes_1,X,Y,L.*ones(1,length(X)),'k');
%                 plot3(app.UIAxes_1,X,Y,zeros(1,length(X)),'k');
%                 hold(app.UIAxes_1,'off');

                MajR = app.Dim1MajorRXEditField.Value;
                MinR = app.Dim2MinorRYEditField.Value;
                L = app.Dim3LengthZEditField.Value;
                
                [X,Y,Z] = deal([]);
                
                for z = 0:app.StepSize*app.StepRoughness:L
                    for r = MajR:-app.StepSize*app.StepRoughness:MinR
                        for theta = 0:app.StepSize*app.StepRoughness/r:2*pi
                            X = [X r*cos(theta)];
                            Y = [Y r*sin(theta)];
                            Z = [Z z];
                        end
                    end
                end
                scatter3(app.UIAxes_1,X,Y,Z,app.MarkerSize,'filled');
            else
%                 W = app.Dim1MajorRXEditField.Value;
%                 D = app.Dim2MinorRYEditField.Value;
%                 H = app.Dim3LengthZEditField.Value;
%                 
%                 a = -pi:pi/2:pi; % Define Corners
%                 ph = pi/4; % Define Angular Orientation (‘Phase’)
%                 x = W.*[cos(a+ph); cos(a+ph)]/cos(ph);
%                 y = D.*[sin(a+ph); sin(a+ph)]/sin(ph);
%                 z = (H/2).*[-ones(size(a)); ones(size(a))]+H/2;
% 
%                 surf(app.UIAxes_1,x,y,z,'FaceColor','g','FaceAlpha',0.7); % Plot Cube
%                 hold(app.UIAxes_1,'on');
%                 patch(app.UIAxes_1,x',y',z','g','FaceAlpha',0.7); % Make Cube Appear Solid
%                 hold(app.UIAxes_1,'off');
                W = app.Dim1MajorRXEditField.Value;
                D = app.Dim2MinorRYEditField.Value;
                H = app.Dim3LengthZEditField.Value; 
                
                [X,Y,Z] = deal([]);
                
                for z = 0:app.StepSize*app.StepRoughness:H
                    for w = W:-2*app.StepSize*app.StepRoughness:0
                        X = [X w/2.*ones(1,length(-w/2:app.StepSize*app.StepRoughness:w/2)),...
                            w/2:-app.StepSize*app.StepRoughness:-w/2,...
                            -w/2.*ones(1,length(w/2:-app.StepSize*app.StepRoughness:-w/2)),...
                            -w/2:app.StepSize*app.StepRoughness:w/2];
                        Y = [Y -w/2:app.StepSize*app.StepRoughness:w/2,...
                            w/2.*ones(1,length(w/2:-app.StepSize*app.StepRoughness:-w/2)),...
                            w/2:-app.StepSize*app.StepRoughness:-w/2,...
                            -w/2.*ones(1,length(-w/2:app.StepSize*app.StepRoughness:w/2))];
                        Z = [Z z.*ones(1,length(Y)-length(Z))];
                    end
                end
                scatter3(app.UIAxes_1,X,Y,Z,app.MarkerSize,'filled');
            end
        end

        % Value changed function: ConnectButton
        function ConnectChanged(app, event)
            value = app.ConnectButton.Value;
            if(value==1)
                try
                    app.Machine = serialport("COM13",9600);
                    app.Machine.Timeout = 5;
%                     configureCallback(app.Machine,"terminator",@readSerialCallback)
                catch
                    app.ConnectButton.Value = 0;
                end
            else
                app.Machine = [];
            end
%             function readSerialCallback(src,evt)
%                 disp(readline(src));
%             end
        end

        % Value changed function: Dim3LengthZEditField
        function Dim3Changed(app, event)
            if(app.TubeButton.Value)
%                 MajR = app.Dim1MajorRXEditField.Value;
%                 MinR = app.Dim2MinorRYEditField.Value;
%                 L = app.Dim3LengthZEditField.Value;
%                 
%                 [X,Y,Z] = cylinder(app.UIAxes_1,MajR,200);
%                 surf(app.UIAxes_1,X,Y,L.*Z,'EdgeColor','none','FaceAlpha',0.55,'FaceColor',[0,1,0]);
%                 hold(app.UIAxes_1,'on');
%                 plot3(app.UIAxes_1,X,Y,L.*ones(1,length(X)),'k');
%                 plot3(app.UIAxes_1,X,Y,zeros(1,length(X)),'k');
%                 [X,Y,Z] = cylinder(app.UIAxes_1,MinR,200);
%                 surf(app.UIAxes_1,X,Y,L.*Z,'EdgeColor','none','FaceAlpha',0.7,'FaceColor',[0,.92,0]);
%                 plot3(app.UIAxes_1,X,Y,L.*ones(1,length(X)),'k');
%                 plot3(app.UIAxes_1,X,Y,zeros(1,length(X)),'k');
%                 hold(app.UIAxes_1,'off');

                MajR = app.Dim1MajorRXEditField.Value;
                MinR = app.Dim2MinorRYEditField.Value;
                L = app.Dim3LengthZEditField.Value;
                
                [X,Y,Z] = deal([]);
                
                for z = 0:app.StepSize*app.StepRoughness:L
                    for r = MajR:-app.StepSize*app.StepRoughness:MinR
                        for theta = 0:app.StepSize*app.StepRoughness/r:2*pi
                            X = [X r*cos(theta)];
                            Y = [Y r*sin(theta)];
                            Z = [Z z];
                        end
                    end
                end
                scatter3(app.UIAxes_1,X,Y,Z,app.MarkerSize,'filled');
            else
%                 W = app.Dim1MajorRXEditField.Value;
%                 D = app.Dim2MinorRYEditField.Value;
%                 H = app.Dim3LengthZEditField.Value;
%                 
%                 a = -pi:pi/2:pi; % Define Corners
%                 ph = pi/4; % Define Angular Orientation (‘Phase’)
%                 x = W.*[cos(a+ph); cos(a+ph)]/cos(ph);
%                 y = D.*[sin(a+ph); sin(a+ph)]/sin(ph);
%                 z = (H/2).*[-ones(size(a)); ones(size(a))]+H/2;
% 
%                 surf(app.UIAxes_1,x,y,z,'FaceColor','g','FaceAlpha',0.7); % Plot Cube
%                 hold(app.UIAxes_1,'on');
%                 patch(app.UIAxes_1,x',y',z','g','FaceAlpha',0.7); % Make Cube Appear Solid
%                 hold(app.UIAxes_1,'off');
                W = app.Dim1MajorRXEditField.Value;
                D = app.Dim2MinorRYEditField.Value;
                H = app.Dim3LengthZEditField.Value; 
                
                [X,Y,Z] = deal([]);
                
                for z = 0:app.StepSize*app.StepRoughness:H
                    for w = W:-2*app.StepSize*app.StepRoughness:0
                        X = [X w/2.*ones(1,length(-w/2:app.StepSize*app.StepRoughness:w/2)),...
                            w/2:-app.StepSize*app.StepRoughness:-w/2,...
                            -w/2.*ones(1,length(w/2:-app.StepSize*app.StepRoughness:-w/2)),...
                            -w/2:app.StepSize*app.StepRoughness:w/2];
                        Y = [Y -w/2:app.StepSize*app.StepRoughness:w/2,...
                            w/2.*ones(1,length(w/2:-app.StepSize*app.StepRoughness:-w/2)),...
                            w/2:-app.StepSize*app.StepRoughness:-w/2,...
                            -w/2.*ones(1,length(-w/2:app.StepSize*app.StepRoughness:w/2))];
                        Z = [Z z.*ones(1,length(Y)-length(Z))];
                    end
                end
                scatter3(app.UIAxes_1,X,Y,Z,app.MarkerSize,'filled');
            end
        end
        
        % Button pushed function: JumpButton
        function JumpPushed(app, event)
            xval = app.XEditField.Value;
            yval = app.YEditField.Value;
            zval = app.ZEditField.Value;
            numXSteps = round((xval-app.Location(1))/app.StepSize);
            numYSteps = round((yval-app.Location(2))/app.StepSize); 
            numZSteps = round((zval-app.Location(3))/app.StepSize); 
            writeline(app.Machine,"X"+num2str(numXSteps)+"Y"+num2str(numYSteps)+"Z"+num2str(numZSteps));
            readline(app.Machine);
            app.Location = app.Location+app.StepSize.*[numXSteps,numYSteps,numZSteps];
            disp(app.Location);
        end

        % Button pushed function: SetHomeButton
        function SetHomePushed(app, event)
            app.Home = app.Location;
        end

        % Button pushed function: SetZeroButton
        function SetZeroPushed(app, event)
            app.Zero = app.Location;
        end

        % Changes arrangement of the app based on UIFigure width
        function updateAppLayout(app, event)
            currentFigureWidth = app.UIFigure.Position(3);
            if(currentFigureWidth <= app.onePanelWidth)
                % Change to a 2x1 grid
                app.GridLayout.RowHeight = {480, 480};
                app.GridLayout.ColumnWidth = {'1x'};
                app.RightPanel.Layout.Row = 2;
                app.RightPanel.Layout.Column = 1;
            else
                % Change to a 1x2 grid
                app.GridLayout.RowHeight = {'1x'};
                app.GridLayout.ColumnWidth = {259, '1x'};
                app.RightPanel.Layout.Row = 1;
                app.RightPanel.Layout.Column = 2;
            end
        end
    end

    % Component initialization
    methods (Access = private)

        % Create UIFigure and components
        function createComponents(app)
            
            app.Zero = app.StepSize.*[480,615,0];

            % Create UIFigure and hide until all components are created
            app.UIFigure = uifigure('Visible', 'off');
            app.UIFigure.AutoResizeChildren = 'off';
            app.UIFigure.Position = [100 100 640 480];
            app.UIFigure.Name = '3D Magnetic Field Probe';
            app.UIFigure.SizeChangedFcn = createCallbackFcn(app, @updateAppLayout, true);

            % Create GridLayout
            app.GridLayout = uigridlayout(app.UIFigure);
            app.GridLayout.ColumnWidth = {259, '1x'};
            app.GridLayout.RowHeight = {'1x'};
            app.GridLayout.ColumnSpacing = 0;
            app.GridLayout.RowSpacing = 0;
            app.GridLayout.Padding = [0 0 0 0];
            app.GridLayout.Scrollable = 'on';

            % Create LeftPanel
            app.LeftPanel = uipanel(app.GridLayout);
            app.LeftPanel.Layout.Row = 1;
            app.LeftPanel.Layout.Column = 1;

            % Create GoZeroButton
            app.GoZeroButton = uibutton(app.LeftPanel, 'push');
            app.GoZeroButton.ButtonPushedFcn = createCallbackFcn(app, @GoZeroPushed, true);
            app.GoZeroButton.Position = [167 416 69 22];
            app.GoZeroButton.Text = 'Go Zero';

            % Create RunningLampLabel
            app.RunningLampLabel = uilabel(app.LeftPanel);
            app.RunningLampLabel.HorizontalAlignment = 'right';
            app.RunningLampLabel.Position = [148 30 50 22];
            app.RunningLampLabel.Text = 'Running';

            % Create RunningLamp
            app.RunningLamp = uilamp(app.LeftPanel);
            app.RunningLamp.Position = [213 30 20 20];
            app.RunningLamp.Color = [0.8 0.8 0.8];

            % Create StepSizeSliderLabel
            app.StepSizeSliderLabel = uilabel(app.LeftPanel);
            app.StepSizeSliderLabel.HorizontalAlignment = 'right';
            app.StepSizeSliderLabel.Position = [17 361 62 32];
            app.StepSizeSliderLabel.Text = 'Step Size';

            % Create StepSizeSlider
            app.StepSizeSlider = uislider(app.LeftPanel);
            app.StepSizeSlider.Limits = [0 1];
            app.StepSizeSlider.MajorTicks = [0 0.5 1];
            app.StepSizeSlider.ValueChangedFcn = createCallbackFcn(app, @SliderChanged, true);
            app.StepSizeSlider.Position = [102 386 133 3];

            % Create GoButton
            app.GoButton = uibutton(app.LeftPanel, 'push');
            app.GoButton.ButtonPushedFcn = createCallbackFcn(app, @GoPushed, true);
            app.GoButton.BackgroundColor = [0 1 0];
            app.GoButton.Position = [25 30 50 22];
            app.GoButton.Text = 'Go';

            % Create StopButton
            app.StopButton = uibutton(app.LeftPanel, 'push');
            app.StopButton.ButtonPushedFcn = createCallbackFcn(app, @StopPushed, true);
            app.StopButton.Interruptible = 'off';
            app.StopButton.BackgroundColor = [1 0.4588 0.4588];
            app.StopButton.Position = [86 30 50 22];
            app.StopButton.Text = 'Stop';

            % Create XButton
            app.XButton = uibutton(app.LeftPanel, 'push');
            app.XButton.ButtonPushedFcn = createCallbackFcn(app, @PXPushed, true);
            app.XButton.Position = [120 293 36 22];
            app.XButton.BackgroundColor = [0.8314 1 0.9843];
            app.XButton.Text = '+X';

            % Create XButton_2
            app.XButton_2 = uibutton(app.LeftPanel, 'push');
            app.XButton_2.ButtonPushedFcn = createCallbackFcn(app, @NXPushed, true);
            app.XButton_2.Position = [50 293 36 22];
            app.XButton_2.BackgroundColor = [0.8314 1 0.9843];
            app.XButton_2.Text = '-X';

            % Create YButton
            app.YButton = uibutton(app.LeftPanel, 'push');
            app.YButton.ButtonPushedFcn = createCallbackFcn(app, @PYPushed, true);
            app.YButton.Position = [85 314 36 22];
            app.YButton.BackgroundColor = [0.8314 1 0.9843];
            app.YButton.Text = '+Y';

            % Create YButton_2
            app.YButton_2 = uibutton(app.LeftPanel, 'push');
            app.YButton_2.ButtonPushedFcn = createCallbackFcn(app, @NYPushed, true);
            app.YButton_2.Position = [85 272 36 22];
            app.YButton_2.BackgroundColor = [0.8314 1 0.9843];
            app.YButton_2.Text = '-Y';

            % Create ZButton
            app.ZButton = uibutton(app.LeftPanel, 'push');
            app.ZButton.ButtonPushedFcn = createCallbackFcn(app, @PZPushed, true);
            app.ZButton.Position = [165 314 36 22];
            app.ZButton.BackgroundColor = [0.8314 1 0.9843];
            app.ZButton.Text = '+Z';

            % Create ZButton_2
            app.ZButton_2 = uibutton(app.LeftPanel, 'push');
            app.ZButton_2.ButtonPushedFcn = createCallbackFcn(app, @NZPushed, true);
            app.ZButton_2.Position = [165 272 36 22];
            app.ZButton_2.BackgroundColor = [0.8314 1 0.9843];
            app.ZButton_2.Text = '-Z';

            % Create ConnectButton
            app.ConnectButton = uibutton(app.LeftPanel, 'state');
            app.ConnectButton.ValueChangedFcn = createCallbackFcn(app, @ConnectChanged, true);
            app.ConnectButton.Text = 'Connect';
            app.ConnectButton.BackgroundColor = [0.8549 1 0.5804];
            app.ConnectButton.Position = [25 439 69 22];

            % Create GoHomeButton
            app.GoHomeButton = uibutton(app.LeftPanel, 'push');
            app.GoHomeButton.ButtonPushedFcn = createCallbackFcn(app, @GoHomePushed, true);
            app.GoHomeButton.Position = [96 416 69 22];
            app.GoHomeButton.Text = 'Go Home';

            % Create Dim3LengthZEditField_2Label
            app.Dim3LengthZEditField_2Label = uilabel(app.LeftPanel);
            app.Dim3LengthZEditField_2Label.HorizontalAlignment = 'right';
            app.Dim3LengthZEditField_2Label.Position = [93 78 102 22];
            app.Dim3LengthZEditField_2Label.Text = 'Dim 3. (Length, Z)';

            % Create Dim3LengthZEditField
            app.Dim3LengthZEditField = uieditfield(app.LeftPanel, 'numeric');
            app.Dim3LengthZEditField.ValueChangedFcn = createCallbackFcn(app, @Dim3Changed, true);
            app.Dim3LengthZEditField.Position = [203 78 41 22];

            % Create Dim2MinorRYEditField_2Label
            app.Dim2MinorRYEditField_2Label = uilabel(app.LeftPanel);
            app.Dim2MinorRYEditField_2Label.HorizontalAlignment = 'right';
            app.Dim2MinorRYEditField_2Label.Position = [93 107 105 22];
            app.Dim2MinorRYEditField_2Label.Text = 'Dim 2. (MinorR, Y)';

            % Create Dim2MinorRYEditField
            app.Dim2MinorRYEditField = uieditfield(app.LeftPanel, 'numeric');
            app.Dim2MinorRYEditField.ValueChangedFcn = createCallbackFcn(app, @Dim2Changed, true);
            app.Dim2MinorRYEditField.Position = [203 107 41 22];

            % Create Dim1MajorRXEditField_2Label
            app.Dim1MajorRXEditField_2Label = uilabel(app.LeftPanel);
            app.Dim1MajorRXEditField_2Label.HorizontalAlignment = 'right';
            app.Dim1MajorRXEditField_2Label.Position = [93 136 105 22];
            app.Dim1MajorRXEditField_2Label.Text = 'Dim 1. (MajorR, X)';

            % Create Dim1MajorRXEditField
            app.Dim1MajorRXEditField = uieditfield(app.LeftPanel, 'numeric');
            app.Dim1MajorRXEditField.ValueChangedFcn = createCallbackFcn(app, @Dim1Changed, true);
            app.Dim1MajorRXEditField.Position = [203 136 41 22];

            % Create GeometryButtonGroup
            app.GeometryButtonGroup = uibuttongroup(app.LeftPanel);
            app.GeometryButtonGroup.SelectionChangedFcn = createCallbackFcn(app, @GeometryChanged, true);
            app.GeometryButtonGroup.Title = 'Geometry';
            app.GeometryButtonGroup.Position = [18 82 65 72];

            % Create TubeButton
            app.TubeButton = uiradiobutton(app.GeometryButtonGroup);
            app.TubeButton.Text = 'Tube';
            app.TubeButton.Position = [11 26 58 22];
            app.TubeButton.Value = true;

            % Create BoxButton
            app.BoxButton = uiradiobutton(app.GeometryButtonGroup);
            app.BoxButton.Text = 'Box';
            app.BoxButton.Position = [11 4 65 22];

            % Create XEditFieldLabel
            app.XEditFieldLabel = uilabel(app.LeftPanel);
            app.XEditFieldLabel.HorizontalAlignment = 'center';
            app.XEditFieldLabel.Position = [26 206 45 22];
            app.XEditFieldLabel.Text = 'X (mm)';

            % Create XEditField
            app.XEditField = uieditfield(app.LeftPanel, 'numeric');
            app.XEditField.HorizontalAlignment = 'center';
            app.XEditField.Position = [25 185 45 22];

            % Create YEditFieldLabel
            app.YEditFieldLabel = uilabel(app.LeftPanel);
            app.YEditFieldLabel.HorizontalAlignment = 'center';
            app.YEditFieldLabel.Position = [78 206 45 22];
            app.YEditFieldLabel.Text = 'Y (mm)';

            % Create YEditField
            app.YEditField = uieditfield(app.LeftPanel, 'numeric');
            app.YEditField.HorizontalAlignment = 'center';
            app.YEditField.Position = [78 185 45 22];

            % Create ZEditFieldLabel
            app.ZEditFieldLabel = uilabel(app.LeftPanel);
            app.ZEditFieldLabel.HorizontalAlignment = 'center';
            app.ZEditFieldLabel.Position = [132.5 206 44 22];
            app.ZEditFieldLabel.Text = 'Z (mm)';
            
            % Create JumpToGlobalLabel
            app.JumpToGlobalLabel = uilabel(app.LeftPanel);
            app.JumpToGlobalLabel.HorizontalAlignment = 'center';
            app.JumpToGlobalLabel.FontWeight = 'bold';
            app.JumpToGlobalLabel.Position = [79 229 103 22];
            app.JumpToGlobalLabel.Text = 'Jump To (Global)';

            % Create ZEditField
            app.ZEditField = uieditfield(app.LeftPanel, 'numeric');
            app.ZEditField.HorizontalAlignment = 'center';
            app.ZEditField.Position = [132 185 45 22];
            
            % Create JumpButton
            app.JumpButton = uibutton(app.LeftPanel, 'push');
            app.JumpButton.ButtonPushedFcn = createCallbackFcn(app, @JumpPushed, true);
            app.JumpButton.BackgroundColor = [1 1 1];
            app.JumpButton.Position = [187 185 46 22];
            app.JumpButton.Text = 'Jump';

            % Create SetHomeButton
            app.SetHomeButton = uibutton(app.LeftPanel, 'push');
            app.SetHomeButton.ButtonPushedFcn = createCallbackFcn(app, @SetHomePushed, true);
            app.SetHomeButton.Position = [96 439 69 22];
            app.SetHomeButton.Text = 'Set Home';

            % Create SetZeroButton
            app.SetZeroButton = uibutton(app.LeftPanel, 'push');
            app.SetZeroButton.ButtonPushedFcn = createCallbackFcn(app, @SetZeroPushed, true);
            app.SetZeroButton.Position = [167 439 69 22];
            app.SetZeroButton.Text = 'Set Zero';

            % Create RightPanel
            app.RightPanel = uipanel(app.GridLayout);
            app.RightPanel.Layout.Row = 1;
            app.RightPanel.Layout.Column = 2;

            % Create UIAxes_1
            app.UIAxes_1 = uiaxes(app.RightPanel);
            title(app.UIAxes_1, 'Scanning Geometry')
            xlabel(app.UIAxes_1, 'X (mm)')
            ylabel(app.UIAxes_1, 'Y (mm)')
            zlabel(app.UIAxes_1, 'Z (mm)')
            xlim(app.UIAxes_1, [-250,250]);
            ylim(app.UIAxes_1, [-250,250]);
            zlim(app.UIAxes_1, [0,250]);
            app.UIAxes_1.XGrid = 'on';
            app.UIAxes_1.YGrid = 'on';
            app.UIAxes_1.ZGrid = 'on';
            app.UIAxes_1.Position = [34 253 300 185];

            % Create UIAxes_2
            app.UIAxes_2 = uiaxes(app.RightPanel);
            title(app.UIAxes_2, 'Magnetic Vector Field')
            xlabel(app.UIAxes_2, 'X (mm)')
            ylabel(app.UIAxes_2, 'Y (mm)')
            zlabel(app.UIAxes_2, 'Z (mm)')
            app.UIAxes_2.XGrid = 'on';
            app.UIAxes_2.YGrid = 'on';
            app.UIAxes_2.ZGrid = 'on';
            app.UIAxes_2.Position = [34 54 300 185];

            % Show the figure after all components are created
            app.UIFigure.Visible = 'on';
        end
    end

    % App creation and deletion
    methods (Access = public)

        % Construct app
        function app = Interact_exported

            % Create UIFigure and components
            createComponents(app)

            % Register the app with App Designer
            registerApp(app, app.UIFigure)
            
            if nargout == 0
                clear app
            end
        end

        % Code that executes before app deletion
        function delete(app)
            app.Machine = [];
            % Delete UIFigure when app is deleted
            delete(app.UIFigure)
        end
    end
end